--[[
                                G A D G E T S
      -----------------------------------------------------------------
                            wildtide@wildtide.net
                           DoomSprout: Rift Forums 
      -----------------------------------------------------------------
      Gadgets Framework   : v0.1.3
      Project Date (UTC)  : 2012-08-07T01:23:40Z
      File Modified (UTC) : 2012-08-07T01:23:40Z (Wildtide)
      -----------------------------------------------------------------     
--]]

local toc, data = ...
local AddonId = toc.identifier
local TXT = Library.Translate

local initDone = false
local dialog = false

-- Initialiser waits until a Gadget is created before registering event handlers
-- This should be done in all gadgets to save on overhead when no gadget instances exist 
local function Init()
	table.insert(WT.Event.PlayerAvailable, {OnPlayerAvailable, AddonId, "BuffBars_OnPlayerAvailable"})	
	initDone = true
end

local filterPanel = nil

local selUnitToTrack = nil
local labMaxBarCount = nil
local sldMaxBarCount = nil
local radGrowUp = nil
local radGrowDown = nil
local radGroupGrow = nil
local radSortUp = nil
local radSortDown = nil
local radGroupSort = nil
local txtHeading = nil
local chkTooltips = nil
local chkCancel = nil
local colorBuffColour = nil
local colorDebuffColour = nil
local colorBuffBackground = nil
local colorDebuffBackground = nil
local colorBuffText = nil
local colorDebuffText = nil
local labBarSpacing = nil
local sldBarSpacing = nil

local function ConfigDialog(container)

	local tabs = UI.CreateFrame("SimpleTabView", "rfTabs", container)
	tabs:SetPoint("TOPLEFT", container, "TOPLEFT")
	tabs:SetPoint("BOTTOMRIGHT", container, "BOTTOMRIGHT", 0, -32)
	
	local frmConfig = UI.CreateFrame("Frame", "rfConfig", tabs.tabContent)
	local frmConfigInner = UI.CreateFrame("Frame", "bbConfigInner", frmConfig)
	frmConfigInner:SetPoint("TOPLEFT", frmConfig, "TOPLEFT", 12, 12)
	frmConfigInner:SetPoint("BOTTOMRIGHT", frmConfig, "BOTTOMRIGHT", -12, -12)
	
	local frmAppearance = UI.CreateFrame("Frame", "rfMacros", tabs.tabContent)
	local frmAppearanceInner = UI.CreateFrame("Frame", "bbFiltersInner", frmAppearance)
	frmAppearanceInner:SetPoint("TOPLEFT", frmAppearance, "TOPLEFT", 4, 4)
	frmAppearanceInner:SetPoint("BOTTOMRIGHT", frmAppearance, "BOTTOMRIGHT", -4, -4)

	tabs:SetTabPosition("top")
	tabs:AddTab("Configuration", frmConfig)
	tabs:AddTab("Appearance", frmAppearance)	

	selUnitToTrack = WT.Control.Select.Create(frmConfigInner, "Unit to Track:", "player", 
	{ 
		{text = "Player", value = "player"}, 
		{text = "Target", value = "player.target"}, 
		{text = "Target's Target", value = "player.target.target"}, 
		{text = "Focus", value = "focus"}, 
		{text = "Focus's Target", value = "focus.target"}, 
		{text = "Pet", value = "player.pet"}, 
		{text = "Pet's Target", value = "player.pet.target"}, 
	})
		
	labMaxBarCount = UI.CreateFrame("Text", "labMaxBarCount", frmConfigInner)
	sldMaxBarCount = UI.CreateFrame("SimpleSlider", "sldMaxBarCount", frmConfigInner)
	radGrowUp = UI.CreateFrame("SimpleRadioButton", "radGrowUp", frmConfigInner)
	radGrowDown = UI.CreateFrame("SimpleRadioButton", "radGrowDown", frmConfigInner)
	radSortUp = UI.CreateFrame("SimpleRadioButton", "radSortUp", frmConfigInner)
	radSortDown = UI.CreateFrame("SimpleRadioButton", "radSortDown", frmConfigInner)
	txtHeading = UI.CreateFrame("RiftTextfield", "txtHeading", frmConfigInner)
	local labHeading = UI.CreateFrame("Text", "labHeading", frmConfigInner)
	chkTooltips = UI.CreateFrame("SimpleCheckbox", "chkTooltips", frmConfig)
	chkCancel = UI.CreateFrame("SimpleCheckbox", "chkCancel", frmConfig)
	labBarSpacing = UI.CreateFrame("Text", "labBarSpacing", frmAppearanceInner)
	sldBarSpacing = UI.CreateFrame("SimpleSlider", "sldBarSpacing", frmAppearanceInner)
	
	radGroupGrow = Library.LibSimpleWidgets.RadioButtonGroup("radGroupGrow")
	radGroupGrow:AddRadioButton(radGrowUp)
	radGroupGrow:AddRadioButton(radGrowDown)

	radGroupSort = Library.LibSimpleWidgets.RadioButtonGroup("radGroupSort")
	radGroupSort:AddRadioButton(radSortUp)
	radGroupSort:AddRadioButton(radSortDown)

	labMaxBarCount:SetText("Maximum Number of Bars:")
	radGrowUp:SetText("Grow Up")
	radGrowDown:SetText("Grow Down")
	radSortUp:SetText("Ascending")
	radSortDown:SetText("Descending")
	txtHeading:SetText("")
	labHeading:SetText("Heading:");
	chkTooltips:SetText("Show Tooltips");
	chkCancel:SetText("Right Click to Cancel");
	labBarSpacing:SetText("Bar Spacing:")
	
	chkTooltips:SetChecked(true)
	chkCancel:SetChecked(true)

	radGrowDown:SetSelected(true)
	radSortDown:SetSelected(true)

	sldMaxBarCount:SetRange(1, 50)
	sldMaxBarCount:SetPosition(10, true)

	sldBarSpacing:SetRange(0, 10)
	sldBarSpacing:SetPosition(3, true)

	selUnitToTrack:SetPoint("TOPLEFT", frmConfigInner, "TOPLEFT", 0, 0)

	labMaxBarCount:SetPoint("TOP", chkTooltips, "BOTTOM", nil, 16) 
	labMaxBarCount:SetPoint("LEFT", frmConfigInner, "LEFT")
	sldMaxBarCount:SetPoint("TOPLEFT", labMaxBarCount, "BOTTOMLEFT", 0, 0) 
	sldMaxBarCount:SetWidth(220)

	radGrowUp:SetPoint("TOP", sldMaxBarCount, "TOP")
	radGrowUp:SetPoint("LEFT", frmConfigInner, "CENTERX")
	radGrowDown:SetPoint("TOPLEFT", radGrowUp, "TOPLEFT", 100, 0)

	radSortUp:SetPoint("TOPLEFT", radGrowUp, "BOTTOMLEFT", 0, 4)
	radSortDown:SetPoint("TOPLEFT", radGrowDown, "BOTTOMLEFT", 0, 4)

	labHeading:SetPoint("TOPLEFT", selUnitToTrack, "BOTTOMLEFT", 0, 4)
	
	txtHeading:SetPoint("TOP", labHeading, "TOP")
	txtHeading:SetPoint("LEFT", labHeading, "RIGHT")
	txtHeading:SetBackgroundColor(0.2, 0.2, 0.2, 1.0)

	chkTooltips:SetPoint("TOPLEFT", labHeading, "BOTTOMLEFT", 0, 4)
	chkCancel:SetPoint("TOPLEFT", chkTooltips, "TOPRIGHT", 8, 0)

	filterPanel = data.CreateBuffFilterPanel(frmConfigInner)
	filterPanel.frmConfigInner:SetPoint("TOPLEFT", sldMaxBarCount, "BOTTOMLEFT", 0, 28)
	filterPanel.frmConfigInner:SetPoint("BOTTOMRIGHT", frmConfigInner, "BOTTOMRIGHT")

	local lblBuffColour = UI.CreateFrame("Text", "lblBuffColour", frmAppearanceInner)
	lblBuffColour:SetText("Buff Color:")
	lblBuffColour:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 4, 10)

	colorBuffColour = WT.CreateColourPicker(frmAppearanceInner, 0.2,0.2,0.6,1.0)
	colorBuffColour:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 100, 10)

	local lblDebuffColour = UI.CreateFrame("Text", "lblBuffColour", frmAppearanceInner)
	lblDebuffColour:SetText("Debuff Color:")
	lblDebuffColour:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 4, 36)

	colorDebuffColour = WT.CreateColourPicker(frmAppearanceInner,0.6,0.1,0.1,1)
	colorDebuffColour:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 100, 36)

	local lblBuffBackground = UI.CreateFrame("Text", "lblBuffBackground", frmAppearanceInner)
	lblBuffBackground:SetText("Buff Background:")
	lblBuffBackground:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 160, 10)

	colorBuffBackground = WT.CreateColourPicker(frmAppearanceInner, 0.2,0.2,0.6,0.3)
	colorBuffBackground:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 280, 10)

	local lblDebuffBackground = UI.CreateFrame("Text", "lblBuffBackground", frmAppearanceInner)
	lblDebuffBackground:SetText("Debuff Background:")
	lblDebuffBackground:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 160, 36)

	colorDebuffBackground = WT.CreateColourPicker(frmAppearanceInner,0.6,0.1,0.1,0.3)
	colorDebuffBackground:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 280, 36)

	local lblBuffText = UI.CreateFrame("Text", "lblBuffText", frmAppearanceInner)
	lblBuffText:SetText("Buff Text:")
	lblBuffText:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 340, 10)

	colorBuffText = WT.CreateColourPicker(frmAppearanceInner, 1,1,1,1)
	colorBuffText:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 420, 10)

	local lblDebuffText = UI.CreateFrame("Text", "lblBuffText", frmAppearanceInner)
	lblDebuffText:SetText("Debuff Text:")
	lblDebuffText:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 340, 36)

	colorDebuffText = WT.CreateColourPicker(frmAppearanceInner, 1,1,1,1)
	colorDebuffText:SetPoint("TOPLEFT", frmAppearanceInner, "TOPLEFT", 420, 36)

	labBarSpacing:SetPoint("TOPLEFT", lblDebuffColour, "BOTTOMLEFT", 0, 16) 
	sldBarSpacing:SetPoint("TOPLEFT", labBarSpacing, "BOTTOMLEFT", 0, 0) 
	sldBarSpacing:SetWidth(220)

end

local function GetConfiguration()

	local config = {}
	config.unitSpec = selUnitToTrack:GetText()

	filterPanel:WriteToConfiguration(config)

	config.maxBars = sldMaxBarCount:GetPosition()

	config.tooltips = chkTooltips:GetChecked()
	config.cancel = chkCancel:GetChecked()
	config.sortDescending = radSortDown:GetSelected()

	config.barSpacing = sldBarSpacing:GetPosition()
	
	config.buffFontColour = { colorBuffText:GetColor() }
	config.buffColour = { colorBuffColour:GetColor() }
	config.buffBackground = { colorBuffBackground:GetColor() }
	
	config.debuffFontColour = { colorDebuffText:GetColor() }
	config.debuffColour = { colorDebuffColour:GetColor() }
	config.debuffBackground = { colorDebuffBackground:GetColor() }
	
	config.heading = txtHeading:GetText()
	config.headingFontColour = {1,1,1,1}
	
	if radGrowUp:GetSelected() then
		config.grow = "up"
	else
		config.grow = "down"
	end
	
	return config 
end

local function SetConfiguration(config)

	selUnitToTrack:SetText(config.unitSpec)
	txtHeading:SetText(config.heading or "")

	chkTooltips:SetChecked(WT.Utility.ToBoolean(config.tooltips)) 
	chkCancel:SetChecked(WT.Utility.ToBoolean(config.cancel))
	 
	if config.sortDescending then
		radSortDown:SetSelected(true)
	else
		radSortUp:SetSelected(true)
	end

	colorBuffColour:SetColor(unpack(config.buffColour))
	colorBuffBackground:SetColor(unpack(config.buffBackground))
	colorBuffText:SetColor(unpack(config.buffFontColour))

	colorDebuffColour:SetColor(unpack(config.debuffColour))
	colorDebuffBackground:SetColor(unpack(config.debuffBackground))
	colorDebuffText:SetColor(unpack(config.debuffFontColour))

	filterPanel:ReadFromConfiguration(config)

	sldMaxBarCount:SetPosition(config.maxBars or 10) 
	sldBarSpacing:SetPosition(config.barSpacing) 

	if config.grow == "up" then
		radGrowUp:SetSelected(true)
	else
		radGrowDown:SetSelected(true)
	end
	
end

local function OnPlayerAvailable()
	print("BuffBars player available triggered")
end

if WT.VersionCheck("Gadgets", 0, 3, 78) then
	WT.Gadget.RegisterFactory("BuffBars",
	{
		name="Buff Bars",
		description="Buff Bars Gadget",
		author="Wildtide",
		version="1.0.0",
		iconTexAddon = AddonId,
		iconTexFile = "img/wtBuffBars.png",
		["Create"] = WT.Gadget.ConfigureBuffBars,
		["Reconfigure"] = WT.Gadget.ConfigureBuffBars,
		["ConfigDialog"] = ConfigDialog,
		["GetConfiguration"] = GetConfiguration, 
		["SetConfiguration"] = SetConfiguration, 
	})
end
