local toc, data = ...
local AddonId = toc.identifier

-- Local config options ---------------------------------------------------------
local simpleRaidFrameWidth = 130
local simpleRaidFrameTopBarHeight = 36
local simpleRaidFrameBottomBarHeight = 12
local simpleRaidFrameMargin = 2
local simpleRaidFrameHeight = simpleRaidFrameTopBarHeight + simpleRaidFrameBottomBarHeight 
---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local SimpleRaidFrame = WT.UnitFrame:Template("SimpleRaidFrame")
SimpleRaidFrame.Configuration.Name = "Simple Raid Frame"
SimpleRaidFrame.Configuration.RaidSuitable = true
SimpleRaidFrame.Configuration.FrameType = "Frame"
SimpleRaidFrame.Configuration.Width = simpleRaidFrameWidth + 3
SimpleRaidFrame.Configuration.Height = simpleRaidFrameHeight + 3
SimpleRaidFrame.Configuration.Resizable = { simpleRaidFrameWidth + 3, simpleRaidFrameHeight + 3, 300, 100 }
---------------------------------------------------------------------------------

-- Override the buff filter to hide some buffs ----------------------------------
local buffPriorities = 
{
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
}
function SimpleRaidFrame:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end
---------------------------------------------------------------------------------

function SimpleRaidFrame:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=0.8,
				color={r=0,g=0,b=0,a=0.6}, colorBinding="aggroColor",
			}, 
			{
				-- Generic Element Configuration
				id="frameBlocked", type="Frame", parent="frameBackdrop", layer=15, visibilityBinding="blockedOrRanged",
				color={r=0,g=0,b=0},alpha=0.6,
				attach = 
				{ 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=3, offsetY=3 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-3, offsetY=-3 } 
				},
			}, 
			{
				-- Generic Element Configuration
				id="barResource", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="BOTTOMLEFT", element="frame", targetPoint="BOTTOMLEFT", offsetX=3, offsetY=-3 },
					{ point="RIGHT", element="frame", targetPoint="RIGHT", offsetX=-3 },
				},
				-- visibilityBinding="id",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="resourcePercent", height=simpleRaidFrameBottomBarHeight, colorBinding="resourceColor",
				texAddon=AddonId, texFile="img/Smooth.png",
				backgroundColor={r=0, g=0, b=0, a=1}
			},
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=3, offsetY=3 },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="TOPRIGHT" },
				},
				-- visibilityBinding="id",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", height=raidFrameTopBarHeight, colorBinding="callingColor",
				texAddon=AddonId, texFile="img/Smooth.png", 
				backgroundColor={r=0, g=0, b=0, a=1}
			},
			{
				-- Generic Element Configuration
				id="frameColor", type="Frame", parent="frameBackdrop", layer=15, alpha=0.3,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=3, offsetY=3 },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="TOPRIGHT" }
				},
				colorBinding="callingColor", 
			},
			{
				-- Generic Element Configuration
				id="frameColor2", type="Frame", parent="frameBackdrop", layer=15, alpha=0.3,
				attach = { 
					{ point="TOPLEFT", element="barHealth", targetPoint="BOTTOMLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-3, offsetY=-3 }
				},
				colorBinding="resourceColor", 
			},
			{
				id="barAbsorb", type="Bar", parent="frameBackdrop", layer=17,
				attach = {
					{ point="TOPLEFT", element="barResource", targetPoint="TOPLEFT", offsetX=0, offsetY=-11 },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="TOPRIGHT" }
				},
				growthDirection="right", 
				binding="absorbPercent",
				color={r=0.08, g=0.91, b=1, a=0.9},
				backgroundColor={r=0.08, g=0.91, b=1, a=0}
			},
			{
				-- Generic Element Configuration
				id="labelHealth", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="TOPRIGHT", element="barHealth", targetPoint="TOPRIGHT", offsetX=1, offsetY=-1 }},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{healthPercent}%", default="", color={r=1,g=1,b=1,a=1}, fontSize=12
			},
			{
				-- Generic Element Configuration
				id="labelHealthR", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barHealth", targetPoint="CENTERRIGHT", offsetX=-1, offsetY=8 }},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{health}/{healthMax}", default="", color={r=1,g=1,b=1,a=1}, fontSize=12
			},
			{
				id="labelHealthAndAbsorbPercent", type="Label", parent="frame", layer=40,
				attach = {{ point="CENTERLEFT", element="barAbsorb", targetPoint="CENTERLEFT", offsetX=3, offsetY=1 }},
				visibilityBinding="id", text="{healthAbsorbPercent}%", fontSize=10,
				color={r=0, g=0, b=0, a=1},
			},
			{
				id="imgRole", type="ImageSet", parent="frameBackdrop", layer=20,
				attach = {{ point="TOPLEFT", element="barHealth", targetPoint="TOPLEFT", offsetX=2, offsetY=2.5 }},
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/Roles12.png", nameBinding="role", cols=1, rows=5, 
				names = { ["tank"] = 0, ["heal"] = 1, ["dps"] = 2, ["support"] = 3 }, defaultIndex = "hide"
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="TOPLEFT", element="imgRole", targetPoint="TOPRIGHT", offsetX=0, offsetY=-5 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{nameMini}", default="", color={r=1,g=1,b=1,a=1}, fontSize=14,
			},
			{
				-- Generic Element Configuration
				id="labelresource", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERLEFT", element="barResource", targetPoint="CENTERLEFT" }},
				visibilityBinding="resource",
				-- Type Specific Element Configuration
				text=" {resource}/{resourceMax}", default="", color={r=1,g=1,b=1,a=1}, fontSize=10
			},
			{
				-- Generic Element Configuration
				id="labelresourceR", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barResource", targetPoint="CENTERRIGHT" }},
				visibilityBinding="resource",
				-- Type Specific Element Configuration
				text="{resourcePercent}%", default="", color={r=1,g=1,b=1,a=1}, fontSize=10
			},
			{
				-- Generic Element Configuration
				id="labelMark", type="Label", parent="frameBackdrop", layer=20,
				attach = {{ point="CENTERLEFT", element="frame", targetPoint="CENTERLEFT", offsetX=2, offsetY=2 }},
				visibilityBinding="mark",alpha=1,
				-- Type Specific Element Configuration
				text="{mark}", default="X", fontSize=20
			},
			{
				id="imgMark", type="MediaSet", parent="frameBackdrop", layer=30,
				attach = {{ point="CENTER", element="frameBackdrop", targetPoint="CENTER", offsetX=20, offsetY=0 }},
				width = 32, height = 32,
				nameBinding="mark",
				names = 
				{
						["1"] = "riftMark01",
						["2"] = "riftMark02",
						["3"] = "riftMark03",
						["4"] = "riftMark04",
						["5"] = "riftMark05",
						["6"] = "riftMark06",
						["7"] = "riftMark07",
						["8"] = "riftMark08",
						["9"] = "riftMark09",
						["10"] = "riftMark10",
						["11"] = "riftMark11",
						["12"] = "riftMark12",
						["13"] = "riftMark13",
						["14"] = "riftMark14",
						["15"] = "riftMark15",
						["16"] = "riftMark16",
						["17"] = "riftMark17",
				},
				visibilityBinding="mark",alpha=1.0,
			},
			{
				-- Generic Element Configuration
				id="imgReady", type="ImageSet", parent="frame", layer=60,
				attach = {{ point="CENTER", element="frame", targetPoint="CENTER" }},
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/wtReady.png", nameBinding="readyStatus", cols=1, rows=2, 
				names = { ["ready"] = 0, ["notready"] = 1 }, defaultIndex = "hide",
			},
			{
				id="buffPanelDebuffs", type="BuffPanel", parent="frame", layer=90,
				attach = {{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=-1 }},
				rows=1, cols=5, iconSize=20, iconSpacingHorizontal=1, iconSpacingVertical=1, borderThickness=1, 
				acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=false, acceptHighPriorityBuffs=false, acceptCriticalPriorityBuffs=false,
				acceptLowPriorityDebuffs=true, acceptMediumPriorityDebuffs=true, acceptHighPriorityDebuffs=true, acceptCriticalPriorityDebuffs=true,
				growthDirection = "left_up", borderColor={r=1,g=1,b=1,a=1},
				sweepOverlay=false,
			},
			
		}
	}
	
	for idx,element in ipairs(template.elements) do
		local showElement = true
		if options.excludeBuffs and element.type=="BuffPanel" then showElement = false end
		if not options.showAbsorb and (element.id=="barAbsorb" or element.id=="labelHealthAndAbsorbPercent") then 
			showElement = false 
		end
		if showElement then
			self:CreateElement(element)
		end 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end
