--[[
File: Interface/List.lua
Author: Zormus
--]]

local Interface = Library.LibZormus.Interface
local _, Roles = ...
Roles.Interface = Roles.Interface or {}
Roles.Interface.List = setmetatable({}, Interface.Frame)
local List = Roles.Interface.List
List.__index = List

List.type = "Frame"

function List:Initialize()
	self.count = 0
	self.items = {}
	self.margin = 4
	self.itemHeight = 0
end

local function UpdateItemHeight(self, index)
	self.items[index]:SetPoint("TOP", self, "TOP", nil, (index - 1) * self.itemHeight + self.margin)
	self.items[index]:SetHeight(self.itemHeight)
end

local function UpdateListHeight(self)
	self:SetHeight(self.itemHeight * self.count + self.margin * 2)
end

function List:SetItemHeight(height)
	self.itemHeight = height
	for index = 1, #self.items do
		UpdateItemHeight(self, index)
	end
	UpdateListHeight(self)
end

local function OnLeftClick(self)
	if self:GetParent().OnItemClick then
		self:GetParent().OnItemClick(self, "Left")
	end
end

local function OnRightClick(self)
	if self:GetParent().OnItemClick then
		self:GetParent().OnItemClick(self, "Right")
	end
end

local function OnHighlight(self, highlight)
	if self:GetParent().OnItemHighlight then
		self:GetParent().OnItemHighlight(self, highlight)
	end
end

local function CreateItem(self, index)
	self.items[index] = Interface.Create("Frame", "Item" .. index, self)
	self.items[index]:SetPoint("LEFT", self, "LEFT", self.margin, nil)
	self.items[index]:SetPoint("RIGHT", self, "RIGHT", -self.margin, nil)
	UpdateItemHeight(self, index)
	self.items[index].OnHighlight = OnHighlight
	self.items[index]:EnableHighlight(true)
	self.items[index].OnLeftClick = OnLeftClick
	self.items[index]:EnableClick(true, "Left")
	self.items[index].OnRightClick = OnRightClick
	self.items[index]:EnableClick(true, "Right")
	self.items[index]:SetSecureMode("restricted")
	if self.InitializeItem then
		self.InitializeItem(self.items[index])
	end
end

function List:GetItem(index)
	if not self.items[index] then
		for i = #self.items + 1, index do
			CreateItem(self, i)
		end
	end
	return self.items[index]
end

function List:SetCount(count)
	self.count = count
	for index, item in ipairs(self.items) do
		item:SetVisible(index <= self.count)
	end
	UpdateListHeight(self)
end
