--[[
File: Interface/Textfield.lua
Author: Zormus
--]]

local Interface = Library[(...).identifier].Interface
Interface.Textfield = setmetatable({}, Interface.Frame)
local Textfield = Interface.Textfield
Textfield.__index = Textfield

Textfield.type = "RiftTextfield"

local function OnTextChange(self)
	if self.maxLines then
		local lines = 1
		local text = self:GetText()
		local position = string.find(text, "\r")
		while position do
			lines = lines + 1
			if lines > self.maxLines then
				break
			end
			position = string.find(text, "\r", position + 1)
		end
		if position then
			self:SetText(string.sub(text, 1, position - 1))
		end
	end
end

local function OnKeyType(self, _, key)
	if key == "\r" then
		local text = self:GetText()
		local cursor = self:GetCursor()
		self:SetText(string.format("%s\r%s", string.sub(text, 1, cursor), string.sub(text, cursor + 1)))
		self:SetCursor(cursor + 1)
		OnTextChange(self)
	end
end

function Textfield:Initialize()
	self.lines = 1
	self:EventAttach(Event.UI.Input.Key.Type, OnKeyType, "KeyType")
	self:EventAttach(Event.UI.Textfield.Change, OnTextChange, "TextChange")
end

function Textfield:SetMaximumLines(maximum)
	self.maxLines = maximum
end

local function Trim(text)
	local char = string.sub(text, 1, 1)
	while char == " " or char == "\r" do
		text = string.sub(text, 2)
		char = string.sub(text, 1, 1)
	end
	char = string.sub(text, string.len(text))
	while char == " " or char == "\r" do
		text = string.sub(text, 1, string.len(text) - 1)
		char = string.sub(text, string.len(text))
	end
	return text
end

function Textfield:GetLines()
	local text = Trim(self:GetText())
	local lines = {}
	while string.len(text) > 0 do
		local line
		local position = string.find(text, "\r")
		if position then
			line = string.sub(text, 1, position - 1)
			text = string.sub(text, position + 1)
		else
			line = text
			text = ""
		end
		line = Trim(line)
		if string.len(line) > 0 then
			table.insert(lines, line)
		end
	end
	return lines
end

function Textfield:SetLines(lines)
	local count = #lines
	if self.maxLines then
		count = math.min(count, self.maxLines)
	end
	if count > 0 then
		local text
		for i = 1, count do
			if i == 1 then
				text = lines[i]
			else
				text = string.format("%s\r%s", text, lines[i])
			end
		end
		self:SetText(text)
	else
		self:SetText("")
	end
end
