------------------------------------------------------------------------------
------------------------ Super Meter - Data Processor ------------------------
------------------------------------------------------------------------------

function SuperMeter.Merge(runFunction)
	if not SuperMeter_enabled then return end

	local merge = {allies = {}, combatTime = 0, deathlog = {}, enemies = {}, startCombat = 0, units = {}}

	if runFunction then
		for k,v in pairs (SuperMeter.State.sessions) do
			
			if merge.combatTime and v.combatTime and v.combatTime > 0 then
				merge.combatTime = merge.combatTime + v.combatTime
			end
			
			if merge.startCombat and v.startCombat and v.startCombat > 0 then
				merge.startCombat = merge.startCombat + v.startCombat
			end
			
			for k,v in pairs (v.units) do
				local name = v.name
				local data = {}
				local id = k
				
				for k,v in pairs (merge.units) do
					if v.name == name then
						id = k
					end
				end
				
				if not merge.units[id] then
					for k,v in pairs (v) do
						data[k] = v
					end
					
					data.abilities = {}
					data.combatlog = {}
					merge.units[id] = data
				else
					local unit = merge.units[id]
					local unitData = v
					
					for k,v in pairs (unit) do
						local number = tonumber(v)
						
						if k == "level" then
							number = nil
						end
						
						if number and number > 0 then
							if unit[k] and unitData[k] and unitData[k] > 0 then
								unit[k] = unit[k] + unitData[k]
							end
						end
					end
				end
			end
		end
	end

	if SuperMeter_showall then
		SuperMeter.State.sessionIndex = SuperMeter.State.sessionIndex + 1
		SuperMeter.State.sessions[SuperMeter.State.sessionIndex] = merge
		SuperMeter_count = SuperMeter.State.sessionIndex
		SuperMeter.State.sessionOffset = 0
	elseif SuperMeter_count == SuperMeter.State.sessionIndex then
		table.remove(SuperMeter.State.sessions, SuperMeter.State.sessionIndex)
		SuperMeter.State.sessionIndex = SuperMeter.State.sessionIndex - 1
		SuperMeter.State.sessionOffset = 0
		SuperMeter_count = 0
	end

	SuperMeter.UI.Interface:ChangeSession()
	SuperMeter.UI.Interface:ManagerSessions()
end

function SuperMeter.Log(unit)
	if not SuperMeter_logEnabled then return end
	if not SuperMeter_logShowing then return end

	local session = SuperMeter.Session()
	local target = unit

	local title = ""
	local data = ""

	if SuperMeter_unit then
		for k,v in pairs (session.units) do
			if session.units[k].name == SuperMeter_unit then
				target = session.units[k]
			end
		end
	end

	if not SuperMeter_unit and not unit then
		local Target = Inspect.Unit.Detail("player.target")
		if Target and session.units[Target.id] then
			target = session.units[Target.id]
		end
	end

	if target and target.player then
		for k,v in pairs (target.combatlog) do
			data = data .. v .. "\n"
		end
		SuperMeter_table = target.combatlog
		SuperMeter_unit = tostring(target.name)
		title = tostring(target.name .. " 's Combat Log")
	else
		for k,v in pairs (session.deathlog) do
			data = data .. v .. "\n"
		end
		SuperMeter_unit = nil
		title = "Session Death Log"
		SuperMeter_table = session.deathlog
	end

	SuperMeter.UI.Interface:LoggerText(data)
	SuperMeter.UI.Interface:LoggerTitle(title)
end

function SuperMeter.Ability(params, avoided, missed, absorb, caster, target)
	if not SuperMeter_enabled or not SuperMeter_inCombat then return end

	local session = SuperMeter.Session(true)

	local abilityName = false
	local damageType = false
	local advanced = false
	local event = false
	local owner = false
	local heal = false

	if not SuperMeter_pvp then
		if not params.abilityName and params.abilityNew then
			local ability = Inspect.Ability.New.Detail(params.abilityNew)
			if ability and ability.id and ability.name then
				params.abilityName = ability.name
			end
		end
	end

	if not params.abilityName or params.abilityName == "attack" then
		params.abilityName = "Attack"
	end

	if params.overkill and SuperMeter_combat[params.target] then
		SuperMeter.Combat({[params.target] = false})
	end

	if target and target.player and target.name and not SuperMeter_pvp then
		advanced = true
	end

	if advanced then
		if SuperMeter_logEnabled then
			if params.damage or params.damageAbsorbed or params.heal or params.overkill then
				SuperMeter.Logger(params, caster, target)
			end
		end
		if SuperMeter_converter then
			if params.damageAbsorbed then
				SuperMeter.Absorb(params, target)
			end
		end
	end

	for k,v in pairs (session.units) do
		if params.caster == session.units[k].pet then
			owner = true
		end
	end

	if params.heal or params.overheal then
		heal = true
	end

	if avoided or missed then
		event = true
	end

	if not heal and not event then
		if params.type == "air" then
			damageType = "Air"
		elseif params.type == "death" then
			damageType = "Death"
		elseif params.type == "earth" then
			damageType = "Earth"
		elseif params.type == "fire" then
			damageType = "Fire"
		elseif params.type == "life" then
			damageType = "Life"
		elseif params.type == "water" then
			damageType = "Water"
		elseif not params.type then
			damageType = "Physical"
		end
	end

	if owner and damageType then
		abilityName = params.abilityName .. " (Pet) (" .. damageType .. ")"
	elseif not owner and damageType then
		abilityName = params.abilityName .. " (" .. damageType .. ")"
	elseif owner and avoided then
		abilityName = params.abilityName .. " (Pet) (Avoided)"
	elseif not owner and avoided then
		abilityName = params.abilityName .. " (Avoided)"
	elseif owner and missed then
		abilityName = params.abilityName .. " (Pet) (Missed)"
	elseif not owner and missed then
		abilityName = params.abilityName .. " (Missed)"
	elseif owner and absorb then
		abilityName = params.abilityName .. " (Pet) (Absorb)"
	elseif not owner and absorb then
		abilityName = params.abilityName .. " (Absorb)"
	elseif owner and heal then
		abilityName = params.abilityName .. " (Pet) (Heal)"
	elseif not owner and heal then
		abilityName = params.abilityName .. " (Heal)"
	end

	if not abilityName then return end

	if not caster.abilities[abilityName] then
		caster.abilities[abilityName] = {
			ability = {id = params.abilityNew, name = params.abilityName},
			damage = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageAbsorbed = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageAvoided = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageBlocked = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageDeflected = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageMissed = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageTaken = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			healing = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			healingTaken = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			overHealing = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			overKill = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
		}
	end

	if not target.abilities[abilityName] then
		target.abilities[abilityName] = {
			ability = {id = params.abilityNew, name = params.abilityName},
			damage = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageAbsorbed = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageAvoided = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageBlocked = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageDeflected = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageMissed = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			damageTaken = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			healing = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			healingTaken = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			overHealing = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
			overKill = {count = 0, critCount = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0},
		}
	end

	if caster.abilities[abilityName] then
		local ability = caster.abilities[abilityName]
		
		if event then
			ability.damageMissed.count = ability.damageMissed.count + 1
			ability.damageMissed.value = ability.damageMissed.value + 1
		end
		
		if params.damage then
			ability.damage.value = ability.damage.value + params.damage
			if params.crit then
				ability.damage.critCount = ability.damage.critCount + 1
				if params.damage < ability.damage.critMin or ability.damage.critMin <= 0 then
					ability.damage.critMin = params.damage
				end
				if params.damage > ability.damage.critMax then
					ability.damage.critMax = params.damage
				end
			else
				ability.damage.count = ability.damage.count + 1
				if params.damage < ability.damage.min or ability.damage.min <= 0 then
					ability.damage.min = params.damage
				end
				if params.damage > ability.damage.max then
					ability.damage.max = params.damage
				end
			end
		end
		
		if params.heal then
			ability.healing.value = ability.healing.value + params.heal
			if params.crit then
				ability.healing.critCount = ability.healing.critCount + 1
				if params.heal < ability.healing.critMin or ability.healing.critMin <= 0 then
					ability.healing.critMin = params.heal
				end
				if params.heal > ability.healing.critMax then
					ability.healing.critMax = params.heal
				end
			else
				ability.healing.count = ability.healing.count + 1
				if params.heal < ability.healing.min or ability.healing.min <= 0 then
					ability.healing.min = params.heal
				end
				if params.heal > ability.healing.max then
					ability.healing.max = params.heal
				end
			end
		end
		
		if params.overheal then
			ability.overHealing.value = ability.overHealing.value + params.overheal
			if params.crit then
				ability.overHealing.critCount = ability.overHealing.critCount + 1
				if params.overheal < ability.overHealing.critMin or ability.overHealing.critMin <= 0 then
					ability.overHealing.critMin = params.overheal
				end
				if params.overheal > ability.overHealing.critMax then
					ability.overHealing.critMax = params.overheal
				end
			else
				ability.overHealing.count = ability.overHealing.count + 1
				if params.overheal < ability.overHealing.min or ability.overHealing.min <= 0 then
					ability.overHealing.min = params.overheal
				end
				if params.overheal > ability.overHealing.max then
					ability.overHealing.max = params.overheal
				end
			end
		end
		
		if params.overkill then
			ability.overKill.value = ability.overKill.value + params.overkill
			if params.crit then
				ability.overKill.critCount = ability.overKill.critCount + 1
				if params.overkill < ability.overKill.critMin or ability.overKill.critMin <= 0 then
					ability.overKill.critMin = params.overkill
				end
				if params.overkill > ability.overKill.critMax then
					ability.overKill.critMax = params.overkill
				end
			else
				ability.overKill.count = ability.overKill.count + 1
				if params.overkill < ability.overKill.min or ability.overKill.min <= 0 then
					ability.overKill.min = params.overkill
				end
				if params.overkill > ability.overKill.max then
					ability.overKill.max = params.overkill
				end
			end
		end
	end

	if target.abilities[abilityName] then
		local ability = target.abilities[abilityName]
		
		if event then
			ability.damageAvoided.count = ability.damageAvoided.count + 1
			ability.damageAvoided.value = ability.damageAvoided.value + 1
		end
		
		if params.damage then
			ability.damageTaken.value = ability.damageTaken.value + params.damage
			if params.crit then
				ability.damageTaken.critCount = ability.damageTaken.critCount + 1
				if params.damage < ability.damageTaken.critMin or ability.damageTaken.critMin <= 0 then
					ability.damageTaken.critMin = params.damage
				end
				if params.damage > ability.damageTaken.critMax then
					ability.damageTaken.critMax = params.damage
				end
			else
				ability.damageTaken.count = ability.damageTaken.count + 1
				if params.damage < ability.damageTaken.min or ability.damageTaken.min <= 0 then
					ability.damageTaken.min = params.damage
				end
				if params.damage > ability.damageTaken.max then
					ability.damageTaken.max = params.damage
				end
			end
		end
		
		if params.heal then
			ability.healingTaken.value = ability.healingTaken.value + params.heal
			if params.crit then
				ability.healingTaken.critCount = ability.healingTaken.critCount + 1
				if params.heal < ability.healingTaken.critMin or ability.healingTaken.critMin <= 0 then
					ability.healingTaken.critMin = params.heal
				end
				if params.heal > ability.healingTaken.critMax then
					ability.healingTaken.critMax = params.heal
				end
			else
				ability.healingTaken.count = ability.healingTaken.count + 1
				if params.heal < ability.healingTaken.min or ability.healingTaken.min <= 0 then
					ability.healingTaken.min = params.heal
				end
				if params.heal > ability.healingTaken.max then
					ability.healingTaken.max = params.heal
				end
			end
		end
		
		if params.damageAbsorbed then
			ability.damageAbsorbed.value = ability.damageAbsorbed.value - params.damageAbsorbed
			if params.crit then
				ability.damageAbsorbed.critCount = ability.damageAbsorbed.critCount + 1
				if params.damageAbsorbed < ability.damageAbsorbed.critMin or ability.damageAbsorbed.critMin <= 0 then
					ability.damageAbsorbed.critMin = -params.damageAbsorbed
				end
				if params.damageAbsorbed > ability.damageAbsorbed.critMax then
					ability.damageAbsorbed.critMax = -params.damageAbsorbed
				end
			else
				ability.damageAbsorbed.count = ability.damageAbsorbed.count + 1
				if params.damageAbsorbed < ability.damageAbsorbed.min or ability.damageAbsorbed.min <= 0 then
					ability.damageAbsorbed.min = -params.damageAbsorbed
				end
				if params.damageAbsorbed > ability.damageAbsorbed.max then
					ability.damageAbsorbed.max = -params.damageAbsorbed
				end
			end
		end
		
		if params.damageBlocked then
			ability.damageBlocked.value = ability.damageBlocked.value + params.damageBlocked
			if params.crit then
				ability.damageBlocked.critCount = ability.damageBlocked.critCount + 1
				if params.damageBlocked < ability.damageBlocked.critMin or ability.damageBlocked.critMin <= 0 then
					ability.damageBlocked.critMin = params.damageBlocked
				end
				if params.damageBlocked > ability.damageBlocked.critMax then
					ability.damageBlocked.critMax = params.damageBlocked
				end
			else
				ability.damageBlocked.count = ability.damageBlocked.count + 1
				if params.damageBlocked < ability.damageBlocked.min or ability.damageBlocked.min <= 0 then
					ability.damageBlocked.min = params.damageBlocked
				end
				if params.damageBlocked > ability.damageBlocked.max then
					ability.damageBlocked.max = params.damageBlocked
				end
			end
		end
		
		if params.damageDeflected then
			ability.damageDeflected.value = ability.damageDeflected.value + params.damageDeflected
			if params.crit then
				ability.damageDeflected.critCount = ability.damageDeflected.critCount + 1
				if params.damageDeflected < ability.damageDeflected.critMin or ability.damageDeflected.critMin <= 0 then
					ability.damageDeflected.critMin = params.damageDeflected
				end
				if params.damageDeflected > ability.damageDeflected.critMax then
					ability.damageDeflected.critMax = params.damageDeflected
				end
			else
				ability.damageDeflected.count = ability.damageDeflected.count + 1
				if params.damageDeflected < ability.damageDeflected.min or ability.damageDeflected.min <= 0 then
					ability.damageDeflected.min = params.damageDeflected
				end
				if params.damageDeflected > ability.damageDeflected.max then
					ability.damageDeflected.max = params.damageDeflected
				end
			end
		end
	end

	if SuperMeter_instance then
		if BossActive[params.targetName] and not SuperMeter_bossActive then
			if target.damageAvoided >= 8 or target.damageIn >= 8 or target.healingIn >= 8 then
				print(params.targetName .. SuperMeter_region15)
				SuperMeter_bossActive = true
				SuperMeter_split = true
			end
			SuperMeter.Combat()
		end
	end
end

function SuperMeter.Absorb(params, target)
	if not SuperMeter_enabled or not SuperMeter_inCombat then return end

	local absorb = -params.damageAbsorbed
	local buffs = Inspect.Buff.List(params.target)
	local session = SuperMeter.Session(true)

	if not SuperMeter_ignore[params.target] then
		SuperMeter_ignore[params.target] = {}
	end

	local unit = SuperMeter_ignore[params.target]
	local runFunction = true
	local caster = false

	if not absorb or not buffs or not unit or not runFunction then return end

	for k,v in pairs (buffs) do
		if runFunction and not unit[k] then
			local detail = Inspect.Buff.Detail(params.target, k)
			if detail and detail.name and Absorbs[detail.name] then
				if detail.caster and session.units[detail.caster] then
					caster = session.units[detail.caster]
				end
				if caster and caster.player then
					local ability = {ability = k, abilityName = detail.name, caster = detail.caster, casterName = caster.name, target = params.target, targetName = target.name, heal = absorb}
					
					caster.healingOut = caster.healingOut + 1
					caster.healing = caster.healing + ability.heal
					caster.healingNonCritical = caster.healingNonCritical + 1
					target.healingTakenNonCritical = target.healingTakenNonCritical + 1
					target.healingTaken = target.healingTaken + ability.heal
					target.healingIn = target.healingIn + 1
					
					SuperMeter.Ability(ability, nil, nil, true, caster, target)
					runFunction = nil
				else
					unit[k] = true
				end
			else
				unit[k] = true
			end
		end
	end
end

function SuperMeter.Logger(params, caster, target)
	if not SuperMeter_enabled or not SuperMeter_inCombat then return end

	local data = {absorbed = "", ability = "", blocked = "", caster = "", critical = "", damage = "", damageType = "", deflected = "", heal = "", info = "", intercepted = "", modified = "", overheal = "", overkill = "", target = "", time = ""}
	local targetName = "[" .. target.name .. "] "
	local session = SuperMeter.Session(true)
	local player = SuperMeter.Player

	if caster.name then
		data.caster = caster.name .. "'s "
	end

	if target.name then
		data.target = target.name .. " "
	end

	if target.combatlog then
		if #target.combatlog > 40 then
			table.remove(target.combatlog, 1)
		end
	end

	if params.type == "air" then
		data.damageType = "Air Damage"
	elseif params.type == "death" then
		data.damageType = "Death Damage"
	elseif params.type == "earth" then
		data.damageType = "Earth Damage"
	elseif params.type == "fire" then
		data.damageType = "Fire Damage"
	elseif params.type == "life" then
		data.damageType = "Life Damage"
	elseif params.type == "water" then
		data.damageType = "Water Damage"
	elseif not params.type then
		data.damageType = "Physical Damage"
	end

	if params.abilityName then
		data.ability = params.abilityName
	end

	if params.crit then
		data.critical = " Critically"
	end

	if params.damage then
		data.damage = params.damage .. " "
	end

	if params.damageAbsorbed then
		data.absorbed = " (" .. -params.damageAbsorbed .. " Absorbed)"
	end

	if params.damageBlocked then
		data.blocked = " (" .. params.damageBlocked .. " Blocked)"
	end

	if params.damageDeflected then
		data.deflected = " (" .. params.damageDeflected .. " Deflected)"
	end

	if params.damageIntercepted then
		data.intercepted = " (" .. -params.damageIntercepted.. " Intercepted)"
	end

	if params.damageModified then
		data.modified = " (" .. -params.damageModified.. " Modified)"
	end

	if params.heal then
		data.heal = params.heal
	end

	if params.overheal then
		data.overheal = " (" .. params.overheal .. " Overheal)"
	end

	if params.overkill then
		data.overkill = " (" .. params.overkill .. " Overkill)"
	end

	if session.deathlog or target.combatlog then
		data.time = "[" .. SuperMeter.Time(session.combatTime) .. "] "
	end

	if params.damage or params.damageAbsorbed and target.combatlog then
		local damageData = "[Damage] " .. data.time .. data.caster .. data.ability .. data.critical .. " Hits " .. data.target .. "For ".. data.damage .. data.damageType .. data.absorbed .. data.blocked .. data.deflected .. data.intercepted .. data.modified .. data.overkill .. "."
		table.insert(target.combatlog, damageData)
	end

	if not params.damage and params.heal and target.combatlog then
		local healingData = "[Healing] " .. data.time .. data.caster .. data.ability .. data.critical .. " Heals " .. data.target .. "For ".. data.heal .. data.overheal .. "."
		table.insert(target.combatlog, healingData)
	end

	if params.damage and params.overkill and session.deathlog then
		local deathData =  "[Death] " .. data.time .. targetName .. data.caster .. data.ability .. data.critical .. " Hits " .. data.target .. "For ".. data.damage .. data.damageType .. data.absorbed .. data.blocked .. data.deflected .. data.intercepted .. data.modified .. data.overkill .. "."
		table.insert(session.deathlog, deathData)
		if target.name == player.name then
			print(deathData)
		end
	end

	if SuperMeter_logShowing then
		if SuperMeter_unit == target.name or not SuperMeter_unit then
			SuperMeter.Log()
		end
	end
end

function SuperMeter.Unit(params, returnCaster, returnTarget)
	if not SuperMeter_enabled or not params then return end

	if not SuperMeter_inCombat and not SuperMeter_pvp then
		if params.heal or params.overheal or params.casterName == params.targetName then return end
		
		if params.caster and params.target then
			local lookup = Inspect.Unit.Lookup(params.caster)
			if lookup and SuperMeter.Player and SuperMeter.Session then
				local mouseover = string.match(lookup, "mouseover")
				local target = string.match(lookup, "target")
				local focus = string.match(lookup, "focus")
				
				if not mouseover and not target and not focus then
					local unit = Inspect.Unit.Detail(params.target)
					if lookup == "player" or lookup == "player.pet" or unit and unit.combat and unit.tier == "raid" and unit.health and unit.health > 0 and not Tier[unit.name] and SuperMeter_group then
						SuperMeter.Combat({[params.caster] = true, [params.target] = true})
						SuperMeter_forced = true
					end
				end
			end
		end
	end

	if not SuperMeter_inCombat then return end

	local session = SuperMeter.Session(true)

	local unitName = false
	local unitID = false
	local owner = false
	local unit = false
	local run = false

	if returnCaster then
		unitName = params.casterName
		unitID = params.caster
	end

	if returnTarget then
		unitName = params.targetName
		unitID = params.target
	end

	if unitName and session.units[unitID] then
		if not session.units[unitID].name then
			session.units[unitID] = nil
		end
	end

	if session.units[unitID] then
		return session.units[unitID]
	end

	for k,v in pairs (session.units) do
		if unitName and unitName == session.units[k].name then
			return session.units[k]
		end
		if unitID and unitID == session.units[k].pet then
			return session.units[k]
		end
	end

	if not session.units[unitID] then
		if not SuperMeter_instance and not params.caster then return end
		if not SuperMeter_instance and not params.target then return end
		
		if unitID then
			owner = Inspect.Unit.Detail(unitID .. ".owner")
			unit = Inspect.Unit.Detail(unitID)
		end
		
		if not unit then
			unit = {id = "Unknown"}
		end
		
		if session.units[unit.id] then
			return session.units[unit.id]
		end
		
		for k,v in pairs (session.units) do
			if unit.name and unit.name == session.units[k].name then
				return session.units[k]
			end
			if unit.id and unit.id == session.units[k].pet then
				return session.units[k]
			end
		end
		
		if unit.nameSecondary then
			for k,v in pairs (session.units) do
				local name = string.match(unit.nameSecondary, "%a+")
				if name == session.units[k].name then
					session.units[k].pet = unitID
					return session.units[k]
				end
			end
		end
		
		if not owner and unit.relation == "friendly" and unit.nameSecondary then return end
		if owner then unit = owner end
		
		if not unit.relation or unit.relation == "hostile" or SuperMeter_instance then
			run = true
		end
		
		if SuperMeter_combat[unit.id] or unit.combat and not unit.blocked then
			run = true
		end
		
		if unit.name == "" or unit.name == " " then
			unit.name = nil
		end
		
		if not run then return end
		
		local data = {
			abilities = {},
			calling = unit.calling,
			combatlog = {},
			damage = 0,
			damageAbsorbIn = 0,
			damageAbsorbed = 0,
			damageAvoidIn = 0,
			damageAvoided = 0,
			damageBlockIn = 0,
			damageBlocked = 0,
			damageCritical = 0,
			damageDeflectIn = 0,
			damageDeflected = 0,
			damageIn = 0,
			damageMissed = 0,
			damageMitigable = 0,
			damageNonCritical = 0,
			damageOut = 0,
			damageTaken = 0,
			damageTakenCritical = 0,
			damageTakenNonCritical = 0,
			healing = 0,
			healingCritical = 0,
			healingIn = 0,
			healingNonCritical = 0,
			healingOut = 0,
			healingTaken = 0,
			healingTakenCritical = 0,
			healingTakenNonCritical = 0,
			level = unit.level,
			name = unit.name,
			overHealing = 0,
			overHealingCritical = 0,
			overHealingOut = 0,
			overKill = 0,
			overKillCritical = 0,
			overKillNonCritical = 0,
			overKillOut = 0,
			overhealingNonCritical = 0,
			pet = "N/A",
			player = unit.player,
			relation = unit.relation,
		}
		
		if owner then
			data.pet = unitID
		end
		
		if not session.units[unit.id] then
			session.units[unit.id] = data
		end
		
		if unit then
			return session.units[unit.id]
		end
	end
end