------------------------------------------------------------------------------
------------------------ Super Meter - Event Handlers ------------------------
------------------------------------------------------------------------------

function SuperMeter.Buff(unit, buffs)
	if not SuperMeter_enabled or not SuperMeter_converter or not unit or not buffs then return end
	if SuperMeter_inCombat or SuperMeter_pvp then return end

	if not SuperMeter_ignore[unit] then
		SuperMeter_ignore[unit] = {}
	end

	local target = SuperMeter_ignore[unit]

	for k,v in pairs (buffs) do
		if target and not target[k] then
			local detail = Inspect.Buff.Detail(unit, k)
			if detail.name and not Absorbs[detail.name] then
				target[k] = true
			end
		end
	end
end

function SuperMeter.Combat(params)
	if not SuperMeter_enabled then return end

	local pet = Inspect.Unit.Lookup("player.pet")
	local player = Inspect.Unit.Detail("player")
	local current = SuperMeter.Session(true)
	local now = Inspect.Time.Frame()
	local session = Session:New()
	local inCombat = false

	if not params then
		params = {[player.id] = not not player.combat}
	end

	if not SuperMeter_inCombat and current.units[player.id] and player.health <= 0 then
		params = {[player.id] = false}
	end

	if not SuperMeter_instance then
		if session and player.zone then
			local zone = Inspect.Zone.Detail(player.zone)
			
			if zone and zone.type == "instance" then
				SuperMeter_instance = true
			end
			
			for k,v in pairs (params) do
				if not SuperMeter_instance then
					local unit = Inspect.Unit.Detail(k)
					if unit and unit.tier == "raid" and not Tier[unit.name] then
						SuperMeter_instance = true
					end
				end
			end
		end
	end

	if SuperMeter_instance then
		SuperMeter_pvp = nil
		player.pvp = nil
	end

	for k,v in pairs (params) do
		if k == player.id or k == pet or SuperMeter_instance and SuperMeter_group then
			SuperMeter_combat[k] = v
		end
	end

	for k,v in pairs (SuperMeter_combat) do
		if not inCombat and v and Inspect.Unit.Lookup(k) then
			inCombat = true
		end
	end

	if SuperMeter_split then
		if SuperMeter_inCombat then
			SuperMeter_inCombat = nil
			session.combatTime = now - session.startCombat
		end
		SuperMeter_split = nil
	end

	if SuperMeter_force then
		if SuperMeter_inCombat then
			inCombat = nil
		else
			inCombat = true
		end
		SuperMeter_force = nil
	end

	if inCombat then
		if not SuperMeter_inCombat then
			SuperMeter.State.sessionIndex = SuperMeter.State.sessionIndex + 1
			SuperMeter.State.sessionOffset = 0
			SuperMeter.State.lastUpdate = 0
			
			if current.combatTime > 0 and current.combatTime <= 4.80 then
				table.remove(SuperMeter.State.sessions, SuperMeter.State.sessionIndex - 1)
				SuperMeter.State.sessionIndex = SuperMeter.State.sessionIndex - 1
			end
			
			if SuperMeter.State.sessionIndex > 200 then
				SuperMeter.State.sessionIndex = SuperMeter.State.sessionIndex - 1
				table.remove(SuperMeter.State.sessions, 1)
			end
			
			SuperMeter.State.sessions[SuperMeter.State.sessionIndex] = session
			session.startCombat = now
			SuperMeter_inCombat = now
			
			SuperMeter.UI.Interface:ManagerSessions()
			SuperMeter.UI.Interface:ChangeSession()
			SuperMeter.UI.Interface:ResetTime()
		end
		SuperMeter.State.combatTime = now
	end

	if not inCombat then
		if SuperMeter_inCombat then
			SuperMeter_bossActive = nil
			SuperMeter_instance = nil
			SuperMeter_inCombat = nil
			SuperMeter_forced = nil
			SuperMeter_combat = {}
			SuperMeter_ignore = {}
			
			SuperMeter.UI.Interface:ManagerSessions()
			SuperMeter.UI.Interface:ChangeSession()
		end
		SuperMeter.State.combatTime = 0
	end

	if player.pvp and not SuperMeter_pvp then
		print(SuperMeter_region16)
	end

	for k,v in pairs (params) do
		if inCombat and not session.units[k] then
			SuperMeter.Unit({caster = k, target = k}, true)
		end
	end

	if inCombat and SuperMeter_showall then
		SuperMeter_showall = nil
		SuperMeter.Merge()
	end

	if player.pvp then
		SuperMeter_pvp = true
	else
		SuperMeter_pvp = nil
	end
end

function SuperMeter.Damage(params)
	if not SuperMeter_enabled or not params then return end

	local caster = SuperMeter.Unit(params, true)
	local target = SuperMeter.Unit(params, nil, true)

	if not caster or not target then return end

	if params.damage then
		if not Damage[params.abilityName] then
			caster.damageOut = caster.damageOut + 1
			caster.damage = caster.damage + params.damage
		end
		if not DamageTaken[params.abilityName] then
			target.damageIn = target.damageIn + 1
			target.damageTaken = target.damageTaken + params.damage
		end
		if DamageCritical[params.abilityName] then
			caster.damageNonCritical = caster.damageNonCritical + 1
			target.damageTakenNonCritical = target.damageTakenNonCritical + 1
		end
		if not Damage[params.abilityName] and params.crit then
			caster.damageCritical = caster.damageCritical + 1
			target.damageTakenCritical = target.damageTakenCritical + 1
		end
	end

	if params.damageAbsorbed then
		if not DamageAbsorbed[params.abilityName] then
			target.damageAbsorbIn = target.damageAbsorbIn + 1
			target.damageAbsorbed = target.damageAbsorbed - params.damageAbsorbed
		end
	end

	if params.damageBlocked then
		if not DamageBlocked[params.abilityName] then
			target.damageBlockIn = target.damageBlockIn + 1
			target.damageBlocked = target.damageBlocked + params.damageBlocked
		end
	end

	if params.damageDeflected then
		if not DamageDeflected[params.abilityName] then
			target.damageDeflectIn = target.damageDeflectIn + 1
			target.damageDeflected = target.damageDeflected + params.damageDeflected
		end
	end

	if params.overkill then
		if not OverKill[params.abilityName] then
			caster.overKillOut = caster.overKillOut + 1
			caster.overKill = caster.overKill + params.overkill
		end
		if DamageCritical[params.abilityName] then
			caster.overKillNonCritical = caster.overKillNonCritical + 1
		end
		if not OverKill[params.abilityName] and params.crit then
			caster.overKillCritical = caster.overKillCritical + 1
		end
	end

	if not params.damage then
		if not DamageTaken[params.abilityName] then
			target.damageIn = target.damageIn + 1
		end
	end

	if not UnAvoidable[params.abilityName] then
		target.damageAvoidIn = target.damageAvoidIn + 1
	end

	if not UnMitigable[params.abilityName] then
		target.damageMitigable = target.damageMitigable + 1
	end

	SuperMeter.Ability(params, nil, nil, nil, caster, target)
end

function SuperMeter.Death(params)
	if not SuperMeter_enabled or not params then return end

	if SuperMeter_combat[params.target] then
		SuperMeter.Combat({[params.target] = false})
	end
end

function SuperMeter.Dodge(params)
	if not SuperMeter_enabled or not params then return end

	local caster = SuperMeter.Unit(params, true)
	local target = SuperMeter.Unit(params, nil, true)

	if not caster or not target then return end

	if not DamageAvoided[params.abilityName] then
		caster.damageMissed = caster.damageMissed + 1
		target.damageAvoidIn = target.damageAvoidIn + 1
		target.damageAvoided = target.damageAvoided + 1
	end

	SuperMeter.Ability(params, true, nil, nil, caster, target)
end

function SuperMeter.Enter(params)
	if not SuperMeter_enabled then return end

	if not SuperMeter_inCombat then
		SuperMeter.Secure()
	end
end

function SuperMeter.Exit(params)
	if not SuperMeter_enabled then return end

	if SuperMeter_inCombat then
		SuperMeter.Secure()
	end
end

function SuperMeter.Heal(params)
	if not SuperMeter_enabled or not params then return end

	local caster = SuperMeter.Unit(params, true)
	local target = SuperMeter.Unit(params, nil, true)

	if not caster or not target then return end

	if params.heal then
		if not Healing[params.abilityName] then
			caster.healingOut = caster.healingOut + 1
			caster.healing = caster.healing + params.heal
		end
		if not HealingTaken[params.abilityName] then
			target.healingIn = target.healingIn + 1
			target.healingTaken = target.healingTaken + params.heal
		end
		if HealingCritical[params.abilityName] then
			caster.healingNonCritical = caster.healingNonCritical + 1
			target.healingTakenNonCritical = target.healingTakenNonCritical + 1
		end
		if not Healing[params.abilityName] and params.crit then
			caster.healingCritical = caster.healingCritical + 1
			target.healingTakenCritical = target.healingTakenCritical + 1
		end
	end

	if params.overheal then
		if not OverHealing[params.abilityName] then
			caster.overHealingOut = caster.overHealingOut + 1
			caster.overHealing = caster.overHealing + params.overheal
		end
		if HealingCritical[params.abilityName] then
			caster.overhealingNonCritical = caster.overhealingNonCritical + 1
		end
		if not OverHealing[params.abilityName] and params.crit then
			caster.overHealingCritical = caster.overHealingCritical + 1
		end
	end

	SuperMeter.Ability(params, nil, nil, nil, caster, target)
end

function SuperMeter.Miss(params)
	if not SuperMeter_enabled or not params then return end

	local caster = SuperMeter.Unit(params, true)
	local target = SuperMeter.Unit(params, nil, true)

	if not caster or not target then return end

	if not DamageMissed[params.abilityName] then
		caster.damageMissed = caster.damageMissed + 1
		target.damageAvoidIn = target.damageAvoidIn + 1
		target.damageAvoided = target.damageAvoided + 1
	end

	SuperMeter.Ability(params, nil, true, nil, caster, target)
end

function SuperMeter.Parry(params)
	if not SuperMeter_enabled or not params then return end

	local caster = SuperMeter.Unit(params, true)
	local target = SuperMeter.Unit(params, nil, true)

	if not caster or not target then return end

	if not DamageAvoided[params.abilityName] then
		caster.damageMissed = caster.damageMissed + 1
		target.damageAvoidIn = target.damageAvoidIn + 1
		target.damageAvoided = target.damageAvoided + 1
	end

	SuperMeter.Ability(params, true, nil, nil, caster, target)
end

function SuperMeter.Resist(params)
	if not SuperMeter_enabled or not params then return end

	local caster = SuperMeter.Unit(params, true)
	local target = SuperMeter.Unit(params, nil, true)

	if not caster or not target then return end

	if not DamageAvoided[params.abilityName] then
		caster.damageMissed = caster.damageMissed + 1
		target.damageAvoidIn = target.damageAvoidIn + 1
		target.damageAvoided = target.damageAvoided + 1
	end

	SuperMeter.Ability(params, true, nil, nil, caster, target)
end

function SuperMeter.Remove(params)
	if not SuperMeter_enabled or not params then return end

	for k,v in pairs (params) do
		if SuperMeter_combat[k] and not v then
			SuperMeter.Combat({[k] = false})
		end
	end
end

function SuperMeter.Secure(params)
	if not SuperMeter_enabled then return end

	local list = Inspect.Unit.List()
	local units = {}

	for k,v in pairs (list) do
		local detail = Inspect.Unit.Detail(k)
		if detail and detail.id and units then
			units[k] = not not detail.combat
		end
	end

	SuperMeter.Combat(units)
end

function SuperMeter.Update(params)
	if not SuperMeter_enabled then return end
	if not SuperMeter_inCombat then return end

	local session = SuperMeter.Session(true)
	local now = Inspect.Time.Frame()

	if now - SuperMeter.State.lastUpdate >= 1.02 then
		SuperMeter.State.lastUpdate = now
		SuperMeter.UI.Interface:SetTime()
		SuperMeter.UI.Interface:Update()
	end

	if SuperMeter_forced then
		if now - SuperMeter.State.combatTime >= 0.42 then
			SuperMeter_forced = nil
			SuperMeter.Combat()
		end
	end

	if SuperMeter_instance then
		if now - SuperMeter.State.combatTime >= 0.82 then
			SuperMeter.Combat()
		end
	end

	session.combatTime = now - session.startCombat
end

table.insert(Event.Buff.Add, {SuperMeter.Buff, "SuperMeter", "Buff Add"})
table.insert(Event.Combat.Damage, {SuperMeter.Damage, "SuperMeter", "Combat Damage"})
table.insert(Event.Combat.Death, {SuperMeter.Death, "SuperMeter", "Combat Deaths"})
table.insert(Event.Combat.Dodge, {SuperMeter.Dodge, "SuperMeter", "Combat Dodges"})
table.insert(Event.Combat.Heal, {SuperMeter.Heal, "SuperMeter", "Combat Heals"})
table.insert(Event.Combat.Miss, {SuperMeter.Miss, "SuperMeter", "Combat Misses"})
table.insert(Event.Combat.Parry, {SuperMeter.Parry, "SuperMeter", "Combat Parries"})
table.insert(Event.Combat.Resist, {SuperMeter.Resist, "SuperMeter", "Combat Resists"})
table.insert(Event.System.Update.Begin, {SuperMeter.Update, "SuperMeter", "Update UI"})
table.insert(Event.System.Update.End, {SuperMeter.Update, "SuperMeter", "Update UI"})
table.insert(Event.System.Secure.Enter, {SuperMeter.Enter, "SuperMeter", "Secure Enter"})
table.insert(Event.System.Secure.Leave, {SuperMeter.Exit, "SuperMeter", "Secure Exit"})
table.insert(Event.Unit.Detail.Combat, {SuperMeter.Combat, "SuperMeter", "Unit Combat"})
table.insert(Event.Unit.Remove, {SuperMeter.Remove, "SuperMeter", "Unit Remove"})