local addon, private = ...
local PADDING = 10

local function AddTip(self, _tip, _icon)
	table.insert(self.tiptable, {_tip, _icon})
end

local function ClearTips(self)
	self.tiptable = {}
end

local function ResetTips(self, tipcount)
	local newtips = tipcount - #self.tips
	local newrec = 0
	self.tipcount = tipcount

	if newtips ~= 0 then
		for i = 1, newtips do
			table.insert(self.tips, {nil, nil})
			newrec = #self.tips
			self.tips[newrec][1] = UI.CreateFrame("Text", "ttText"..newrec, self)
			self.tips[newrec][1]:SetBackgroundColor(0, 0, 0, 1)
			self.tips[newrec][2]= UI.CreateFrame("Texture", "ttTexture"..newrec, self)
			self.tips[newrec][2]:SetHeight(20)
			self.tips[newrec][2]:SetWidth(20)
		end
	end

	for i = 1, #self.tips do
		self.tips[i][1]:ClearAll()
		self.tips[i][1]:SetVisible(false)
		self.tips[i][2]:ClearAll()
		self.tips[i][2]:SetVisible(false)
	end
end

local function SetTips(self)
	local _right = 0
	self:ResetTips(#self.tiptable)
	--for i, _tip in pairs(_tips) do
	for i=1,self.tipcount do
		self.tips[i][1]:SetText(self.tiptable[i][1])
		self.tips[i][1]:SetVisible(true)
		self.tips[i][2]:SetTexture("UpgradeMe", self.GetTexture(self.tiptable[i][2]))
		self.tips[i][2]:SetVisible(true)

		_pFld = self.tips[i][2]
		if _right < (self.tips[i][1]:GetWidth() + self.tips[i][2]:GetWidth() + (PADDING * 3)) then
			_right = (self.tips[i][1]:GetWidth() + self.tips[i][2]:GetWidth() + (PADDING * 3))
		end
	end
	self:SetWidth(_right)
end

local function Hide(self)
   self:SetVisible(false)
   self.owner = nil
end

local function Show(self)
	local _cur = 0
	local _parent = self:GetParent()
	local _tture = nil
	local _pFld = self.ToolTipFlag
	local _offset = 0

	self:SetTips()
	self.ToolTipFlag:ClearAll()

	if UM_GLOBAL.flgTooltip then
		self.ToolTipFlag:SetPoint("TOPLEFT", self, "TOPLEFT", PADDING, PADDING)
		self.ToolTipFlag:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", -1 * PADDING, PADDING + 20)
		self.ToolTipFlag:SetVisible(true)
		_offset = 20
	else
		self.ToolTipFlag:SetPoint("TOPLEFT", self, "TOPLEFT", PADDING, PADDING)
		self.ToolTipFlag:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", -1 * PADDING, 5)
		self.ToolTipFlag:SetVisible(false)
		_offset = 0
	end

	for i=1, self.tipcount do
		self.tips[i][2]:ClearAll()
		self.tips[i][2]:SetPoint("TOPLEFT", _pFld, "BOTTOMLEFT", 0, 5)
		self.tips[i][1]:ClearAll()
		self.tips[i][1]:SetPoint("CENTERLEFT", self.tips[i][2], "CENTERRIGHT", PADDING, 0)
		_pFld = self.tips[i][2]
	end

	if _offset == 20 then _offset = 25 end
	--self:SetHeight((#self.tips * 20) + ((2 * PADDING)+((#self.tips-1) * 5)) + _offset)
	self:SetHeight((self.tipcount * 20) + ((2 * PADDING)+((self.tipcount-1) * 5)) + _offset)

	self:SetVisible(true)
	_parent:SetStrata("topmost")
	self:SetPoint("BOTTOMCENTER", UI.Native.Tooltip, "BOTTOMCENTER", 0, self:GetHeight() + 5)

end

local function GetTexture(umTexture)
	local _retTexture = nil

	if not umTexture then
		_retTexture = nil
	elseif umTexture == "Up" then
		_retTexture = "Textures/Up.png"
	elseif umTexture == "Down" then
		_retTexture = "Textures/Down.png"
	elseif umTexture == "Side" then
		_retTexture = "Textures/Side.png"
	else
		_retTexture = nil
	end

	return _retTexture
end

local function GetTipWidth(self)
    local _width = 0
	for i, tip in pairs(self.tips) do
		if _width < ((tip[1]:GetWidth() + tip[2]:GetWidth()) + (2 * PADDING)) then
			_width = ((tip[1]:GetWidth() + tip[2]:GetWidth()) + (2 * PADDING))
		end
	end
	return _width
end

local function GetTipHeight(self)
    local _height = 0
	for i, tip in pairs(self.tips) do
		if _height < ((tip[1]:GetHeight() + tip[2]:GetHeight()) + (2 * PADDING)) then
			_height = ((tip[1]:GetHeight() + tip[2]:GetHeight()) + (2 * PADDING))
		end
	end
	return _width
end

function umTooltip(parent, tipcount)
	local _tt = UI.CreateFrame("Frame", "umTooltip", parent)

	_tt.tips = {}
	_tt.tiptable = {}

	_tt.tipcount = 0

	_tt:SetLayer(999)
	if not _tt.__lsw_border then Library.LibSimpleWidgets.SetBorder("tooltip", _tt) end
	_tt.__lsw_border:SetPosition("inside")
	_tt.ToolTipFlag = UI.CreateFrame("Text", "umTooltipFlag", _tt)
	_tt.ToolTipFlag:SetBackgroundColor(.9,.3,.3,.8)
	_tt.ToolTipFlag:SetPoint("TOPLEFT", _tt, "TOPLEFT", PADDING, PADDING)
	_tt.ToolTipFlag:SetPoint("BOTTOMRIGHT", _tt, "TOPRIGHT", -1 * PADDING, PADDING + 20)
	_tt.ToolTipFlag:SetVisible(false)

	_tt.ToolTipFlagText = UI.CreateFrame("Text", "umTooltipFlag", _tt.ToolTipFlag)
	_tt.ToolTipFlagText:SetBackgroundColor(1,.3,.3, 0)
	_tt.ToolTipFlagText:SetFontColor(0, 0, 0, 1)
	_tt.ToolTipFlagText:SetPoint("TOPCENTER", _tt.ToolTipFlag, "TOPCENTER", 0, 0)
	_tt.ToolTipFlagText:SetVisible(true)
	_tt.ToolTipFlagText:SetText("Current Outfit Conflict")


	_tt:SetBackgroundColor(0, 0, 0, 1)
	_tt.Hide = Hide
	_tt.Show = Show
	_tt.SetTips = SetTips
	_tt.GetTipWidth = GetTipWidth
	_tt.GetTipHeight = GetTipHeight
	_tt.GetTexture = GetTexture
	_tt.ResetTips = ResetTips
	_tt.AddTip	= AddTip
	_tt.ClearTips = ClearTips

	_tt:Hide()
	return _tt
end



